/*
 * Author:	gxc
 * Create data:	2005-01-29 20:09
 */

#ifndef TORRENTFILE_H_
#define TORRENTFILE_H_

#include <vector>
#include <string>
#include "BitStorm.h"
#include "bdict.h"

class CTorrentFile : public ITorrentFile
{
public:
	CTorrentFile();
	virtual ~CTorrentFile();
private:
	typedef std::vector<TFileInfo> TFileInfoVector;
	typedef std::vector<std::string> TAnnounceVector;
	
	std::string _encoding;
	bool _utf8Valid;
	
	int _pieceLength;
	std::string _pieces;
	
	TAnnounceVector _announces;
	int _creatDate;
	std::string _comment;
	std::string _creator;
	std::string _topDir;
	std::string _name;
	
	std::string _commentUTF8;
	std::string _creatorUTF8;
	std::string _topDirUTF8;
	std::string _nameUTF8;	
	
	TFileInfoVector  _fileInfos;
	int64_t _totalSize;
	
	unsigned char _infoHash[20];
	
	IBTTask* _task;
private:
	bool parseDict(CBDict& dict);
	bool parseInfoDict(CBDict& dict);		
	void updateUTF8();
public:	
 	void setBTTask(IBTTask* task);
	IBTTask* getBTTask();
		
	bool load(const char* filePath);
	void setCharacterEncoding(const char* enc);
	bool IsUTF8Valid();
	unsigned int getAnnounceCount();
	const char* getAnnounce(unsigned int index);
	int getCreationDate();
	const char* getComment();
	const char* getCreator();
	std::string getName();
	std::string getTopDir();
	const char* getCommentUTF8();
	const char* getCreatorUTF8();
	std::string getNameUTF8();
	std::string getTopDirUTF8();	
	unsigned int getPieceLength();
	unsigned int getPieceCount();
	std::string getPieceHash(unsigned int pieceIndex);	
	unsigned int getFileCount();
	TFileInfo getFileInfo(unsigned int index);
	int64_t getTotalSize();
	const unsigned char* getInfoHash();	
};

#endif /*TORRENTFILE_H_*/
